import os
from ConfigParser import ConfigParser
from datetime import date
from httplib import HTTPConnection
from lxml.etree import Element, SubElement, fromstring, parse, tostring, XSLT
from shutil import rmtree
from StringIO import StringIO
from sys import argv
from syslog import syslog
from urllib import urlopen
from xml.sax.saxutils import escape as xml_escape

# FIXME nsMap uit config gebruiken
nsMap = {
    'ddi'   :"http://www.icpsr.umich.edu/DDI",
    'dii'   :"urn:mpeg:mpeg21:2002:01-DII-NS",
    'didl'  :"urn:mpeg:mpeg21:2002:02-DIDL-NS",
    'dip'   :"urn:mpeg:mpeg21:2005:01-DIP-NS",
    'foaf'  :"http://xmlns.com/foaf/0.1/",
    'oai'   :'http://www.openarchives.org/OAI/2.0/',
    'mods'  :"http://www.loc.gov/mods/v3",
    'nereus':"http://www.nereus4economics.info/",
    'rdf'   :"http://www.w3.org/1999/02/22-rdf-syntax-ns#",
    'srw'   :"http://www.loc.gov/zing/srw/",
    'extra' :"http://meresco.com/namespace/fields/extra",
    'meta'  :"http://meresco.com/namespace/harvester/meta",
}

def getSectionDict(config,  section = ''):
# returns a hash of an entire section
    hash = {}
    options  = config.items(section)
    for option,value in options:
        hash[option] = value
    return hash

class ArchiveMaker():
    def __init__(self, configfile):
        self.config = ConfigParser()
        self.config.read(configfile)
        self._xsltDir = self.config.get('main', 'xsltDir')
        self._xslt = XSLT(parse(open(os.path.join(self._xsltDir, "didlmods2redif.xsl"))))
        self._addHandle = XSLT(parse(open(os.path.join(self._xsltDir, "addRePEcHandle.xsl"))))
        self._docroot = self.config.get('archive', 'docroot')
        self._archive = self.config.get('archive', 'id')
        self._baseURL = self.config.get('partnersinfo', 'baseURL')
        self._maximumRecords = int(self.config.get('reader', 'maximumRecords'))
        try: self._seriesMap = getSectionDict(self.config, 'series')
        except: self._seriesMap = {}
        self._excludeSet = getSectionDict(self.config, 'excludeSet')
        self._tmp = os.path.join(self._docroot, self._archive + "_tmp")
        rmtree(self._tmp, ignore_errors=True)
        os.mkdir(self._tmp)

    def make_metadata_file(self, query, seriesId, idLocation):
        stop = self._maximumRecords
        # FIXME sortering
        #sortBy =  ''
        #sortDescending = True
        baseURL = self.config.get('reader', 'baseURL')
        url = baseURL + \
                "?version=1.1&operation=searchRetrieve&maximumRecords=%s&x-recordSchema=meta&query=%s" %(stop, query)
        s = ""
        diag = ""
        try:
            sruResponse = parse(urlopen(url))
        except IOError, (errno, strerror):
            diag += "ERROR url=%s errno=%s strerror=%s" % (url, errno, strerror)
            return -1, s, diag
        total = int(sruResponse.xpath("//srw:numberOfRecords/text()", nsMap)[0])
            
        if total == 0:
            ## no publications found ##
            diag += "WARNING no records found url=%s " % (url)
            return 0, s, diag
        
        s = "\xEF\xBB\xBF"
        records = sruResponse.xpath("//srw:record", nsMap)
        for record in records:
            didlMods = record.xpath("srw:recordData/oai:metadata/didl:DIDL", nsMap)[0]
            docid = ""
            if idLocation:
                try:
                    docid = record.xpath("srw:recordData/oai:metadata/didl:DIDL//"+ idLocation["location"]+"/text()", nsMap)[0]
                    if "replaceWhat" in idLocation:
                        if "replaceBy" not in idLocation:
                            idLocation["replaceBy"] = ""
                        docid = docid.replace(idLocation["replaceWhat"], idLocation["replaceBy"])
                except: pass
            if not docid:
                try:
                    docid = didlMods.xpath("didl:Item/didl:Descriptor/didl:Statement/dii:Identifier/text()", nsMap)[0].split("#", 1)[0]
                except IndexError:
                    continue
            handle = "RePEc:" + self._archive + ":" + seriesId + ":" + docid
            qhandle = "\'" + handle + "\'"
            redif = self._xslt(didlMods, handle=qhandle)
            s += str(redif)
            s += "\n\n"
            meta = record.xpath('srw:extraRecordData/recordData[@recordSchema="meta"]/meta:meta', nsMap)[0]
            metaHandle = meta.xpath('meta:record/meta:repec/meta:handle/text()', nsMap)
            if not metaHandle or metaHandle[0] != handle:
                qtoday = "\'" + str(date.today()) + "\'"
                newMeta = tostring(self._addHandle(meta, handle=qhandle, today=qtoday))
                repository = meta.xpath("meta:repository/meta:id/text()", nsMap)[0]
                recordIdentifier = meta.xpath("meta:record/meta:id/text()", nsMap)[0]
                document = Document(newMeta, '').get(method="Element")
                recordUpdate = UpdateRequest("replace", repository, recordIdentifier, document)
                #print recordUpdate.get(xml_declaration=True)
                port = self.config.get('writer', 'port')
                self._status, d = recordUpdate.send(port)
                if self._status == 'fail': diag += " failed to write handle " + d
        return total, s, diag

    def make_archive_file(self):
        n = "\n"
        f = "\xEF\xBB\xBF"
        f += "Template-type: "     + "ReDIF-Archive 1.0"                + n
        f += "Handle: RePEc:"     + self._archive                       + n
        f += "Name: "             + self.config.get('archive', 'name')  + n
        f += "Maintainer-Email: " + self.config.get('archive', 'email') + n
        f += "URL: "              + self.config.get('archive', 'url')
        path = os.path.join(self._tmp, self._archive+"arch.rdf")
        file = open(path, 'w')
        file.write(f)
        file.close()
 
    def make_series_file(self, partners):
        path = os.path.join(self._tmp, self._archive+"seri.rdf")
        file = open(path, 'w')
        s = "\xEF\xBB\xBF"
        for key in partners:
            s += """Template-type: ReDIF-Series 1.0
Name: %(title)s
Provider-Name: %(organisation)s
Provider-Homepage: %(homepage)s
Maintainer-Name: %(name)s
Maintainer-Email: %(email)s
Handle: %(handle)s """ %{'title'        :partners[key]['Name'],
                         'organisation' :partners[key]['Provider-Name'],
                         'homepage'     :partners[key]['Provider-Homepage'],
                         'name'         :partners[key]['Maintainer-Name'],
                         'email'        :partners[key]['Maintainer-Email'],
                         'handle'       :partners[key]['Handle'] }
            s += "\n\n"
        file.write(s)
        file.close()
               
    def partners(self):
        # get partner information
        partnersInfo = {}
        partnerslist = parse(self._baseURL + "?request=listPartners")
        partners = partnerslist.xpath("/nereus:partners/nereus:partner", namespaces=nsMap)
        for partner in partners:
            id = partner.xpath("@id", namespaces=nsMap)[0]
            status = partner.xpath('properties/entry[@key="neeo.partner.status"]/text()')[0]
            if status != "active": continue

            url = self._baseURL + "?request=getPartner&partnerId=" + id
            try: partnerInfo = parse(url)
            except: continue
            #print tostring(partnerInfo)
            institutionPath = '/nereus:partner/rdf:RDF/foaf:Organization[rdf:type="info:eu-repo/semantics/institution"]'
            info = {}
            info['partnerId']         = id
            info['Provider-Name']     = partnerInfo.xpath(institutionPath+"/foaf:name/text()",                       nsMap)[0]
            info['Provider-Homepage'] = partnerInfo.xpath(institutionPath+"/foaf:homepage/@rdf:resource",            nsMap)[0]
            info['Maintainer-Email']  = partnerInfo.xpath(institutionPath+"/nereus:contacts/nereus:contact/@e-mail", nsMap)[0]
            info['Name']              = "Open Access publications from " + info['Provider-Name']
            try:
                info['Maintainer-Name'] = partnerInfo.xpath(institutionPath+"/nereus:contacts/nereus:contact/@name", nsMap)[0]
            except:
                info['Maintainer-Name'] = ""
            repoId = partnerInfo.xpath(institutionPath+"/nereus:repositories/rdf:Bag/rdf:li/@rdf:nodeID", nsMap)[0]
            defaultseries = ""
            for key in self._seriesMap:
                if id == self._seriesMap[key]:
                    defaultseries = key
            if defaultseries:
                try:
                    feed_repec_default = \
                        partnerInfo.xpath('/nereus:partner/rdf:RDF/rdf:Description[@rdf:nodeID="%s"][rdf:type="info:eu-repo/semantics/repository"]/nereus:feed-repec-default/text()' %(repoId), nsMap)[0]
                except IndexError:
                    feed_repec_default = "yes"
                if feed_repec_default == "yes":
                    feed_repec_default_excluded_sets = []
                    try:
                        excludedSets = \
                            partnerInfo.xpath('/nereus:partner/rdf:RDF/rdf:Description[@rdf:nodeID="%s"][rdf:type="info:eu-repo/semantics/repository"]/nereus:feed-repec-default-excluded-sets/rdf:Bag/rdf:li' %(repoId), nsMap)
                        for set in excludedSets:
                            feed_repec_default_excluded_sets.append(set.xpath("@rdf:nodeID", nsMap)[0])
                    except IndexError:
                        pass
                    partnersInfo[defaultseries] = info.copy()
                    partnersInfo[defaultseries]['feed-repec-default'] = "yes"
                    partnersInfo[defaultseries]['feed-repec-default-excluded-sets'] = feed_repec_default_excluded_sets
                    partnersInfo[defaultseries]['Handle'] = "RePEc:ner:" + defaultseries
            try:
                feed_repec_series = partnerInfo.xpath('/nereus:partner/rdf:RDF/rdf:Description[@rdf:nodeID="%s"][rdf:type="info:eu-repo/semantics/repository"]/nereus:feed-repec-series/rdf:Bag/rdf:li/@rdf:nodeID' %(repoId), nsMap)
                for series in feed_repec_series:
                    if not series: continue
                    description = '/nereus:partner/rdf:RDF/rdf:Description[@rdf:nodeID="%s"][rdf:type="info:eu-repo/semantics/series"]/nereus:description' %(series)
                    authority, archive, id = series.split(':')
                    if authority != "RePEc" or archive != self._archive:
                        continue
                    try: 
                        name  = partnerInfo.xpath(description+"/nereus:name/text()", nsMap)[0]
                        email = partnerInfo.xpath(description+"/nereus:maintainer-email/text()", nsMap)[0]
                        sets  = partnerInfo.xpath('/nereus:partner/rdf:RDF/rdf:Description[@rdf:nodeID="%s"][rdf:type="info:eu-repo/semantics/series"]/nereus:sets/rdf:Bag/rdf:li/text()' %(series), nsMap)
                    except: continue
                    partnersInfo[id] = info.copy()
                    partnersInfo[id]['feed-repec-default'] = "no"
                    partnersInfo[id]['Name'] = name
                    partnersInfo[id]['Maintainer-Email'] = email
                    partnersInfo[id]['Handle'] = series
                    partnersInfo[id]["sets"] = sets
                    try: partnersInfo[id]["Provider-Name"]   = partnerInfo.xpath(description+"/nereus:publisher-name/text()",  nsMap)[0]
                    except: pass
                    try: partnersInfo[id]["Provider-Email"]  = partnerInfo.xpath(description+"/nereus:publisher-email/text()", nsMap)[0]
                    except: pass
                    try: partnersInfo[id]["Maintainer-Name"] = partnerInfo.xpath(description+"/nereus:maintainer-name/text()", nsMap)[0]
                    except: pass
                    try: partnersInfo[id]["ISSN"]            = partnerInfo.xpath(description+"/nereus:issn/text()",            nsMap)[0]
                    except: pass
                    try: partnersInfo[id]["Restriction"]     = partnerInfo.xpath(description+"/nereus:restriction/text()",     nsMap)[0]
                    except: pass
                    try: partnersInfo[id]["Description"]     = partnerInfo.xpath(description+"/nereus:description/text()",     nsMap)[0]
                    except: pass
                    try: partnersInfo[id]["Keywords"]        = partnerInfo.xpath(description+"/nereus:keywords/text()",        nsMap)[0]
                    except: pass
                    try: partnersInfo[id]["Editor-Name"]     = partnerInfo.xpath(description+"/nereus:editor/text()",          nsMap)[0]
                    except: pass
                    try: partnersInfo[id]["Notification"]    = partnerInfo.xpath(description+"/nereus:notification/text()",    nsMap)[0]
                    except: pass
                    try: partnersInfo[id]["Followup"]        = partnerInfo.xpath(description+"/nereus:followup/text()",        nsMap)[0]
                    except: pass
                    try: partnersInfo[id]["Predecessor"]     = partnerInfo.xpath(description+"/nereus:predecessor/text()",     nsMap)[0]
                    except: pass
                    try:
                        identifier = '/nereus:partner/rdf:RDF/rdf:Description[@rdf:nodeID="%s"][rdf:type="info:eu-repo/semantics/series"]/nereus:identifier' %(series)
                        location = {}
                        location["location"] = partnerInfo.xpath(identifier+"/nereus:location/text()", nsMap)[0]
                        try:
                            location["replaceWhat"] = partnerInfo.xpath(identifier+"/nereus:replace/nereus:what/text()", nsMap)[0]
                            location["replaceBy"] = partnerInfo.xpath(identifier+"/nereus:replace/nereus:by/text()", nsMap)[0]
                        except:pass
                        if location:
                            partnersInfo[id]["idLocation"] = location
                    except:pass
            except: pass
        return partnersInfo

    def make_metadata(self, partners):
        log = open(os.path.join(self._tmp, "repec.log"), "w")
        for series in partners:
            partnerId = partners[series]['partnerId']
            path = os.path.join(self._tmp, series)
            #print "path="+path
            if not os.path.exists(path):
                os.mkdir(path)
            query = partnerId
            if partners[series]['feed-repec-default'] == 'yes':
                query += ' and fulltext.url.exists exact true not mods.genre="info:eu-repo/semantics/workingpaper"'
                if series in self._excludeSet:
                    for set in self._excludeSet[series].split():
                        query += ' not set=' + set
                if 'feed-repec-default-excluded-sets' in partners[series]:
                    for set in partners[series]['feed-repec-default-excluded-sets']:
                        if set: query += ' not set=' + set
            else:
                i = 0
                for set in partners[series]["sets"]:
                    if i == 0: query += " and ("    
                    else: query += " or "
                    query += "set=" + set
                    i += 1
                query += ")"
            log.write("query=" + query + "\n")
            idLocation = {} 
            if 'idLocation' in partners[series]:
               idLocation = partners[series]['idLocation']
            total, result, diag = self.make_metadata_file(query, series, idLocation)
            log.write("total=" + str(total) + "\n")
            if diag: log.write(diag + "\n")
            log.write("\n")
            if total > 0:
                file = open(os.path.join(path, "metadata.rdf"), 'w')
                file.write(result)
                file.close()

# copied from enrichmentserver FIXME: class in separate package; same for the other classes that follow
class Tree(object):
    def __init__(self):
        self.root = None

    def get(self, method="xml", xml_declaration=False):
        if method == "Element":
            return self.root
        elif method == "xml":
            return tostring(self.root, xml_declaration=xml_declaration, encoding="utf-8")

class Document(Tree):
    def __init__(self, meta, record, part=""):
        self.root = Element('document', xmlns="http://meresco.com/namespace/harvester/document")
        SubElement(self.root, 'part', name="meta").text = meta
        if record: SubElement(self.root, 'part', name=part).text = record

# FIXME namespaces nu op twee plekken
diag_ns = 'http://www.loc.gov/zing/srw/diagnostic/'
meta_ns = 'http://meresco.com/namespace/harvester/meta'
mods_ns = 'http://www.loc.gov/mods/v3'
srw_ns  = 'http://www.loc.gov/zing/srw/'
ucp_ns  = 'info:lc/xmlns/update-v1'
ns = {'ddi':'http://www.icpsr.umich.edu/DDI',
      'diag': diag_ns,
      'didl': 'urn:mpeg:mpeg21:2002:02-DIDL-NS',
      'dii' : 'urn:mpeg:mpeg21:2002:01-DII-NS',
      'meta': meta_ns,
      'mods': mods_ns,
      'srw' : srw_ns,
      'ucp' : ucp_ns}

class UpdateRequest(Tree):
    def __init__(self, action, repository, recordIdentifier, document):
        self.ns = {"xmlns:srw": srw_ns, "xmlns:ucp": ucp_ns}
        self.root = Element('srw:updateRequest', self.ns)
        SubElement(self.root, 'srw:version').text = "1.0"
        self.action = SubElement(self.root, 'ucp:action')
        if action == "replace" : self.action.text = "info:srw/action/1/replace"
        elif action == "delete": self.action.text = "info:srw/action/1/delete"
        SubElement(self.root, 'ucp:recordIdentifier').text = repository + ":" + recordIdentifier
        if action == "replace":
            self.record = SubElement(self.root, 'srw:record')
            SubElement(self.record, 'srw:recordPacking').text = 'xml'
            SubElement(self.record, 'srw:recordSchema').text  = 'document'
            SubElement(self.record, 'srw:recordData').append(document)

    def send(self, port):
        # FIXME HTTPExceptions afvangen
        self.request = self.get(xml_declaration=True)
        #print self.request
        ## writer waarnaar de enrichment wordt gestuurd ##
        connection = HTTPConnection('localhost', port)
        connection.putrequest("POST", "/update")
        connection.putheader("Content-Type", "text/xml; charset=\"utf-8\"")
        connection.putheader("Content-Length", str(len(self.request)))
        connection.endheaders()
        connection.send(self.request)

        response = connection.getresponse()
        status = response.status
        content = fromstring(response.read())
        #print tostring(content)
        if status == 200:
            if content.xpath("//ucp:operationStatus/text()", ns)[0] == "success":
                return "success", ""
            else:
                return "fail", content.xpath("//diag:diagnostic/diag:details/text()", ns)[0]
        else:
            return "fail", "HTTP problem: %s" %status

def main():
    args=argv[1:]
    if len(args) != 1:
            print "supply one argument for the config file"
            exit()
    configFile = args[0]
    archive = ArchiveMaker(configFile)
    archive.make_archive_file()
    partners = archive.partners()
    archive.make_series_file(partners)
    archive.make_metadata(partners)
    archive_dir = os.path.join(archive._docroot, archive._archive)
    rmtree(os.path.join(archive_dir))
    os.rename(archive._tmp, archive_dir)
    print "-- finished --"

if __name__ == '__main__':
        main()
